using Dakota.Data;
using Dakota.Space;
using Microsoft.Xna.Framework;
using Nez;

namespace Dakota.Missions
{
    public class Mission1 : SpaceMission
    {
        /*
         * 1) fly into boundary: add asteroids & boundary
         * 2) destroy one asteroid: OK 3 more
         * 3) three more: zone map
         * 4) fly to highlighted zone
         * this is a metroidvania shooter?
         */

        private NoFlyZone _noFlyZone;
        private string _status;
        private const int SmallAsteroidGoal = 16;

        public override string GetMissionStatus()
        {
            return _status;
        }

        public override void OnStart()
        {
            ZoneShiftDisabled = true;
            SpaceScene.CurrentZone = ZoneData.GetZone(16, 13);
            SpaceScene.PlayerWarpTo(new Vector2(500, 500));
            SpaceScene.Primary = PrimaryWeapon.Gun;

            _status = "EXPLORE AREA";

            AddBoundary("inner",
                new Rectangle(100, 100, SpaceScene.CurrentZone.Width - 100, SpaceScene.CurrentZone.Height - 100));
            _noFlyZone = SpaceScene.CreateEntity("no fly zone").AddComponent(new NoFlyZone(100));
            _noFlyZone.Enabled = false;

            Core.Schedule(0.1f,
                timer => SpaceScene.DialogBox.ShowDialog(
                    "THIS IS YOUR FIRST TIME FLYING \nOUTSIDE SIMULATION.\nTRY AND FLY AROUND A BIT.", 10));
        }

        internal void AddEmitters()
        {
            SpaceScene.AsteroidEmitter.EmitSpeed = 5;
            SpaceScene.AsteroidEmitter.AddEmitter(new Vector2(0, 0));
            SpaceScene.AsteroidEmitter.AddEmitter(new Vector2(1000, 1000));
            SpaceScene.AsteroidEmitter.AddEmitter(new Vector2(0, 1000));
            SpaceScene.AsteroidEmitter.AddEmitter(new Vector2(1000, 0));
            SpaceScene.AsteroidEmitter.AddEmitter(new Vector2(500, 0));
            SpaceScene.AsteroidEmitter.AddEmitter(new Vector2(0, 500));
            SpaceScene.AsteroidEmitter.AddEmitter(new Vector2(1000, 500));
            SpaceScene.AsteroidEmitter.AddEmitter(new Vector2(500, 1000));
        }

        public override void OnExitRegion(string name)
        {
            if (!_noFlyZone.Enabled)
            {
                SpaceScene.DialogBox.ShowDialog("IT IS EASY TO GET LOST IN SPACE\nLET'S STAY CLOSE FOR NOW", 3);
                _noFlyZone.Enabled = true;
                _status = "DESTROY ASTEROIDS";
                AddEmitters();
            }

            AutopilotReverse(150);
        }

        public override void OnAsteroidDestroyed(AsteroidSize size)
        {
            base.OnAsteroidDestroyed(size);

            if (size == AsteroidSize.LARGE && AsteroidsDestroyed[AsteroidSize.LARGE] == 1)
            {
                SpaceScene.DialogBox.ShowDialog("GREAT JOB! BE SURE TO BREAK UP THE REST OF IT", 3);
            }
            else if (size == AsteroidSize.MEDIUM && AsteroidsDestroyed[AsteroidSize.MEDIUM] % 3 == 0)
            {
                SpaceScene.DialogBox.ShowDialog("KEEP IT UP!", 3);
            }
            else if (size == AsteroidSize.SMALL)
            {
                if (AsteroidsDestroyed[AsteroidSize.SMALL] >= 4 &&
                    AsteroidsDestroyed[AsteroidSize.SMALL] < SmallAsteroidGoal)
                {
                    var percent = (int) ((float) AsteroidsDestroyed[AsteroidSize.SMALL] / SmallAsteroidGoal * 100);
                    _status = $"DESTROY ASTEROIDS\n{percent}";
                }

                switch (AsteroidsDestroyed[AsteroidSize.SMALL])
                {
                    case 4:
                        SpaceScene.DialogBox.ShowDialog("OK ONE DOWN, PHEW!\nTAKE OUT 3 MORE", 3);
                        break;
                    case 10:
                        SpaceScene.DialogBox.ShowDialog("JUST A FEW MORE!", 3);
                        break;
                    case SmallAsteroidGoal:
                        SpaceScene.DialogBox.ShowDialog("ALL DONE! LET'S LOOK AT YOUR MAP.", 3);
                        break;
                }
            }
        }

        public override void OnPlayerHit()
        {
            base.OnPlayerHit();

            if (PlayerHitCount == 1)
                SpaceScene.DialogBox.ShowDialog("WATCH OUT, THESE SHIPS ARE EXPENSIVE", 3);
            else if (PlayerHitCount == 10)
                SpaceScene.DialogBox.ShowDialog("YOU ARE TERRIBLE AT THIS", 3);
        }
    }
}