using System;
using System.Collections.Generic;
using Dakota.Utils;
using Microsoft.Xna.Framework;
using Nez;
using Random = Nez.Random;

namespace Dakota.Space
{
    public class BaddieEmitter : SceneComponent, IUpdatable
    {
        private List<Vector2> _emitterPoints;
        private float _nextEmission;
        public float EmitSpeed;

        public BaddieEmitter()
        {
            _emitterPoints = new List<Vector2>();
            EmitSpeed = 0;
            _nextEmission = 0;
        }

        public void AddEmitter(Vector2 pos)
        {
            _emitterPoints.Add(pos);
        }

        public void Emit()
        {
            var scene = (SpaceScene) Scene;
            var posIndex = Random.NextInt(_emitterPoints.Count);
            var pos = _emitterPoints[posIndex];
            var angle = DakotaUtils.ToAngle(new Vector2(scene.CurrentZone.Width / 2 - pos.X,
                scene.CurrentZone.Height / 2 - pos.Y));
            var vel = DakotaUtils.VecFromAngle(angle) * (Random.NextFloat(30) + 30);
            scene.AddAsteroid(pos, AsteroidSize.LARGE, vel);
        }

        public override void Update()
        {
            if (EmitSpeed > 0)
            {
                if (_nextEmission == 0)
                    _nextEmission = EmitSpeed;
                _nextEmission -= Time.DeltaTime;
                if (_nextEmission < 0)
                {
                    _nextEmission += EmitSpeed;
                    Emit();
                }
            }
        }
    }
}