using Dakota.Data;
using Microsoft.Xna.Framework;
using Nez;

namespace Dakota.Space
{
    public class GalacticMap : RenderableComponent
    {
        private const float GridSize = 25f;
        public override float Width => 500;
        public override float Height => 520;

        public GalacticMap()
        {
            RenderLayer = GameConstants.UiRenderLayerBg;
        }

        public void FillBox(Batcher batcher, int x, int y, Color color)
        {
            batcher.DrawRect(
                GridSize * x + 1 + Entity.Position.X,
                GridSize * y + 2 + Entity.Position.Y,
                GridSize - 3,
                GridSize - 3,
                color);
        }

        public override void Render(Batcher batcher, Camera camera)
        {
            var xSectors = 26;
            var ySectors = 26;

            // draw horizontal lines
            for (var i = 0; i <= ySectors; i++)
            {
                batcher.DrawLine(
                    0 + Entity.Position.X,
                    i * GridSize + Entity.Position.Y,
                    xSectors * GridSize + Entity.Position.X,
                    i * GridSize + Entity.Position.Y,
                    GameConstants.SpaceUiColor);

                // draw vertical lines
                for (var j = 0; j <= xSectors; j++)
                {
                    batcher.DrawLine(
                        j * GridSize + Entity.Position.X,
                        0 + Entity.Position.Y,
                        j * GridSize + Entity.Position.X,
                        ySectors * GridSize + Entity.Position.Y,
                        GameConstants.SpaceUiColor);

                    if (i < ySectors && j < xSectors)
                    {
                        if (ZoneData.GetZone(j, i).IsCurvedX)
                            FillBox(batcher, j, i, Color.Purple);
                        if (ZoneData.GetZone(j, i).IsCurvedY)
                            FillBox(batcher, j, i, Color.Brown);
                    }
                }
            }

            // color current zone
            var scene = (SpaceScene) Entity.Scene;
            FillBox(batcher, scene.CurrentZone.UniversalX, scene.CurrentZone.UniversalY, Color.White);
        }
    }
}