using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Nez;
using Nez.BitmapFonts;

namespace Lighthouse
{
    public enum DialogState
    {
        Empty,
        Writing,
        Done,
    }

    public class Dialog : RenderableComponent, IUpdatable
    {
        private string _text;
        private int _letterCount;
        private float _nextTime;
        private const float LetterSpeed = 0.1f;
        private const float AutoDisplayTime = 10f;
        private Queue<string> _thoughts;
        private Texture2D _more;
        private DialogState _state;

        public override float Width => 400;
        public override float Height => 100;

        public Dialog()
        {
            _text = "";
            _thoughts = new Queue<string>();
        }

        public override void OnAddedToEntity()
        {
            _more = Graphics.CreateSingleColorTexture(10, 10, Color.White);
        }

        public void AddThought(string text)
        {
            if (_state == DialogState.Empty)
                Think(text);
            else
                _thoughts.Enqueue(text);
        }

        public void Think(string text)
        {
            _text = text;
            _letterCount = 1;
            _state = DialogState.Writing;
            _nextTime = Time.TotalTime + LetterSpeed;
        }

        public void Update()
        {
            // nothing to update
            if (Time.TotalTime < _nextTime && !VirtualInput.Action.IsPressed)
                return;
            
            switch (_state)
            {
                case DialogState.Writing:
                    _nextTime = Time.TotalTime + LetterSpeed;
                    _letterCount += 1;
                    if (_letterCount == _text.Length || VirtualInput.Action.IsPressed)
                    {
                        _letterCount = _text.Length;
                        _state = DialogState.Done;
                        _nextTime = Time.TotalTime + AutoDisplayTime;
                    }
                    break;
                case DialogState.Done:
                    if (_thoughts.Count != 0)
                        Think(_thoughts.Dequeue());
                    else
                        _state = DialogState.Empty;
                    break;
                default:
                    break;
            }
        }

        public override void Render(Batcher batcher, Camera camera)
        {
            if (_state == DialogState.Empty)
                return;
            var s = _text.Substring(0, _letterCount);
            var width = Graphics.Instance.BitmapFont.MeasureString(s).X;
            var pos = new Vector2((Entity.Scene.Camera.Bounds.Width - width)/2, Entity.Position.Y);
            var alpha = 1f;
            if (_state == DialogState.Done)
            {
                alpha = (_nextTime - Time.TotalTime) / AutoDisplayTime;
            }
            batcher.DrawString(Graphics.Instance.BitmapFont, s, pos, Color.White * alpha);
            if (_state == DialogState.Done && _thoughts.Count != 0)
            {
                pos += new Vector2(2 + width, 0);
                batcher.Draw(_more, pos, Color.White * alpha);
            }
        }
    }
}