using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Nez;

namespace Lighthouse
{
    public class MapRenderer : RenderableComponent
    {
        // 640/16 = 40, 360/16=22.5 (half a tile cut off? seems ok)
        private const int VisibleTilesWide = 40;
        private const int VisibleTilesTall = 23;

        private Texture2D _tilesetTexture;
        private readonly World _world;
        // how far from top left corner should player sprite be pinned
        private int _playerXOffset;
        private int _playerYOffset;

        public override float Width => VisibleTilesWide * _tilesetTexture.Width;
        public override float Height => VisibleTilesTall * _tilesetTexture.Height;

        public MapRenderer(World world, int xOffset, int yOffset)
        {
            _world = world;
            LayerDepth = 1f;
            _playerXOffset = xOffset;
            _playerYOffset = yOffset;
        }

        public override void OnAddedToEntity()
        {
            _tilesetTexture = Entity.Scene.Content.Load<Texture2D>("Textures/Ground");
        }

        public override void Render(Batcher batcher, Camera camera)
        {
            var map = _world.Location.Map;
            var dest = new Rectangle(0, 0, map.TileWidth, map.TileHeight);
            var src = new Rectangle(0, 0, map.TileWidth, map.TileHeight);
            var xOffset = _world.Player.X - _playerXOffset;
            var yOffset = _world.Player.Y - _playerYOffset;
            
            foreach (var layer in map.Layers)
            {
                for (int y = yOffset; y < yOffset+VisibleTilesTall; y++)
                {
                    for (int x = xOffset; x < xOffset+VisibleTilesWide; x++)
                    {
                        if (y < 0 || x < 0 || x >= layer.width || y >= layer.height)
                            continue;
                        int gid = layer.data[x + (y * layer.width)];
                        if (gid == 0)
                            continue;
                        var mapTileset = map.GetTiledMapTileset(gid);
                        var tileset = _world.Location.Tilesets[mapTileset.firstgid];
                        var rect = map.GetSourceRect(mapTileset, tileset, gid);

                        dest.X = (x - xOffset) * map.TileWidth;
                        dest.Y = (y - yOffset) * map.TileHeight;
                        src.X = rect.x;
                        src.Y = rect.y;
                        batcher.Draw(_tilesetTexture, dest, src, Color.White);
                    }
                }
            }

        }
    }
}