using Nez;

namespace Lighthouse
{
    public class Player : Component, IUpdatable
    {
        private const double MoveDelaySeconds = 0.05;
        private double _lastMoveTime;

        public PlayerAnimator Animator;
        private World _world;
        // world coordinates
        public int X;
        public int Y;

        public Player(World world)
        {
            _world = world;
        }
        
        public void Update()
        {
            if (Time.TotalTime - _lastMoveTime > MoveDelaySeconds)
            {
                var newX = X;
                var newY = Y;

                if (VirtualInput.Left.IsDown)
                {
                    newX -= 1;
                    Animator.StepAnimation("walk left");
                }
                else if (VirtualInput.Right.IsDown)
                {
                    newX += 1;
                    Animator.StepAnimation("walk right");
                }
                else if (VirtualInput.Up.IsDown)
                {
                    newY -= 1;
                    Animator.StepAnimation("walk up");
                }
                else if (VirtualInput.Down.IsDown)
                {
                    newY += 1;
                    Animator.StepAnimation("walk down");
                }

                if (newX != X || newY != Y)
                {
                    if (_world.GetTileType(newX, newY) == "ground")
                    {
                        _lastMoveTime = Time.TotalTime;
                        SetPlayerPosition(newX, newY);
                    }
                }
            }
        }

        public void SetPlayerPosition(int x, int y)
        {
            foreach (var warp in _world.Location.Warps)
            {
                if (x == warp.FromX && y == warp.FromY)
                {
                    //Console.WriteLine($"{warp.FromX} {warp.FromY} => {warp.ToX} {warp.ToY}");
                    x = warp.ToX;
                    y = warp.ToY;
                    _world.Location = warp.To;
                }
            }
            X = x;
            Y = y;
        }
    }
}