using System;
using System.Collections.Generic;
using System.IO;
using TiledCS;

namespace Lighthouse.Data
{
    public class Locations
    {
        public static Location Island;
        public static Location Cave;
        
        public static void Initialize()
        {
            Island = new Location( "Content/Maps/World.tmx");
            Cave = new Location("Content/Maps/Cave.tmx");
            AddWarp(Island, 421, 269, Cave, 22, 38);
        }

        static void AddWarp(Location from, int fromX, int fromY, Location to, int toX, int toY)
        {
            from.Warps.Add(new Warp(fromX, fromY, to, toX, toY));
            to.Warps.Add(new Warp(toX, toY + 1, from, fromX, fromY + 1));
        }
    }
    
    public struct Warp
    {
        public int FromX;
        public int FromY;
        public Location To;
        public int ToX;
        public int ToY;

        public Warp(int fromX, int fromY, Location to, int toX, int toY)
        {
            FromX = fromX;
            FromY = fromY;
            To = to;
            ToX = toX;
            ToY = toY;
        }
    }
    
    public class Location
    {
        public Location(string tmxPath)
        {
            // be sure to set "Copy to Output Directory" property of these files to `Copy if newer`
            Map = new TiledMap(tmxPath);
            Tilesets = Map.GetTiledTilesets(Path.GetDirectoryName(tmxPath) + "/");

            if (Map.Layers[0].chunks != null)
            {
                Console.WriteLine("ERROR: Chunks still present, make map non-infinite!");
            }
        }

        public TiledMap Map;
        public Dictionary<int, TiledTileset> Tilesets;
        public List<Warp> Warps = new List<Warp>();
    }
}